/**
 * @file pointers.cpp
 * @author Alicia 
 * @ref https://en.cppreference.com/w/cpp/memory/shared_ptr 
 * @date 2021-09-23
 * 
 */
#include <iostream>
#include <memory>
#include <time.h>

class Example{
public:
    Example()
    {
        std::cout << "Created an Example! " << std::endl;
    }

    ~Example(){
        std::cout << "Destroyed an Example! " << std::endl;
    }

    void print_hey(){
        std::cout << "Hey I'm an example! " << std::endl;
    }
};


int main()
{
    // -------- DEMO 1 ------------
    // raw pointer
    // Example* example = new Example();
    // std:: cout << example << std::endl;
    // delete example;

    // -------- DEMO 2 ------------
    // // Copy the pointed-to object by dereferencing the pointer
    // // to access the contents of the memory location.
    // // example2 is a another object, allocated here on the stack
    // Example* example = new Example()
    // Example example2 = *example;

    // // Declare a pointer that points to example using the addressof operator
    // Example* ecopy = &example2;

    
    // -------- DEMO 3 ------------
    // Smart pointer, (also explain scope)
    {
        // unique_ptr constructor explicit
        // std::unique_ptr<Example> example = new Example();
        std::unique_ptr<Example> example2(new Example());
        std::unique_ptr<Example> example3 = std::make_unique<Example>();
        // std::unique_ptr<Example> copy_example = example;
    }


    return 0;
}